import unittest
from CryptoTokenAuth import CryptoToken, encryption
import time


class Test(unittest.TestCase):

    CryptoTokenAuth = CryptoToken(key='testkey')

    def test_create_token(self):
        token_normal = self.CryptoTokenAuth.create()
        token_with_ip = self.CryptoTokenAuth.create(ip='127.0.0.1')

        self.assertEqual(str, type(token_normal))
        self.assertEqual(str, type(token_with_ip))

    def test_authenticate_token(self):
        token = self.CryptoTokenAuth.create()
        is_valid_token = self.CryptoTokenAuth.authenticate(token=token)

        self.assertTrue(is_valid_token)

    def test_token_authentication_check_expire(self):
        token_1 = self.CryptoTokenAuth.create(expire_secs=2)
        time.sleep(2)
        is_valid_token_1 = self.CryptoTokenAuth.authenticate(token=token_1)

        token_2 = self.CryptoTokenAuth.create()
        is_valid_token_2 = self.CryptoTokenAuth.authenticate(token=token_2)

        token_3 = self.CryptoTokenAuth.create(ip='127.0.0.1')
        is_valid_token_3 = self.CryptoTokenAuth.authenticate(token=token_3, ip='127.0.0.1')

        token_4 = self.CryptoTokenAuth.create(ip='127.0.0.1', expire_secs=2)
        time.sleep(2)
        is_valid_token_4 = self.CryptoTokenAuth.authenticate(token=token_4, ip='127.0.0.1')

        self.assertFalse(is_valid_token_1)
        self.assertTrue(is_valid_token_2)
        self.assertTrue(is_valid_token_3)
        self.assertFalse(is_valid_token_4)

    def test_authenticate_token_check_ip(self):
        token_1 = self.CryptoTokenAuth.create(ip='127.0.0.1')
        is_valid_token_1 = self.CryptoTokenAuth.authenticate(token=token_1, ip='127.0.0.1')

        token_2 = self.CryptoTokenAuth.create(ip='127.0.0.1')
        is_valid_token_2 = self.CryptoTokenAuth.authenticate(token=token_2, ip='912.0.2.5')

        token_3 = self.CryptoTokenAuth.create(ip='127.0.0.1', expire_secs=2)
        time.sleep(2)
        is_valid_token_3 = self.CryptoTokenAuth.authenticate(token=token_3, ip='90.12.5.1')

        self.assertTrue(is_valid_token_1)
        self.assertFalse(is_valid_token_2)
        self.assertFalse(is_valid_token_3)

    def test_token_salt_feature(self):
        salt = b'sometestsaltinbytes'
        key = 'sometestkeymeow'

        client_1 = CryptoToken(key=key, salt=salt)
        token_1 = client_1.create()

        client_2 = CryptoToken(key=key, salt=salt)
        is_valid_token = client_2.authenticate(token_1)

        self.assertTrue(is_valid_token)

    def test_check_salt(self):
        salt = encryption.getSalt()

        self.assertTrue(bytes == type(salt))
        self.assertTrue(len(salt) > 50)



import time
import calendar
from CryptoTokenAuth.encryption import is_valid_key, encrypt_content, decrypt_content


class CryptoToken:
    def __init__(self, key=None, salt=None, expire_secs=60):
        self.key = key
        self.expire_secs = expire_secs
        self.salt = salt

    def authenticate(self, token: str, key=None, ip=None, salt=None):
        if not key:
            key = self.key

        if not salt:
            salt = self.salt

        if is_valid_key(key=key, content=token, salt=salt):
            try:
                token_data = decrypt_content(content=token, key=key, salt=salt)
                gmt = time.gmtime()
                current_timestamp = calendar.timegm(gmt)

                if ip:
                    timestamp, token_ip_addr = token_data.split("||")

                    if int(timestamp) > int(current_timestamp) and str(ip) == str(token_ip_addr):
                        return True

                else:
                    timestamp = token_data

                    if int(timestamp) > int(current_timestamp):
                        return True

            except ValueError:
                return False

        return False

    def create(self, key=None, expire_secs=None, ip=None, salt=None):
        if not expire_secs:
            expire_secs = self.expire_secs

        if not key:
            key = self.key

        if not salt:
            salt = self.salt

        gmt = time.gmtime()
        timestamp = calendar.timegm(gmt)

        expire_timestamp = timestamp + expire_secs

        if ip:
            token_data = f"{expire_timestamp}||{ip}"
        else:
            token_data = str(expire_timestamp)

        token = encrypt_content(content=token_data, key=key, salt=salt)

        return token



try:
    import os
    import io
    import string
    import random
    from CryptoTokenAuth.version import VERSION
    from CryptoTokenAuth.authentication import CryptoToken
except ModuleNotFoundError:
    pass

__version__ = VERSION


def setup():
    salt = "".join(random.choice(string.ascii_uppercase + string.digits + string.ascii_lowercase + string.octdigits) for _ in range(100))
    code = f"{salt}"
    file_path = os.path.join(os.path.dirname(__file__), 'salt.key')

    def write_code_to_file(source_code):
        with open(file_path, 'w') as ofh:
            fh = io.StringIO(source_code)
            ofh.writelines(fh.readlines())

    write_code_to_file(code)


if not os.path.isfile("salt.key"):
    setup()



from functools import wraps
from time import time


def processTime(func):
    @wraps(func)
    def _time_it(*args, **kwargs):
        start = int(round(time() * 1000))
        try:
            return func(*args, **kwargs)
        finally:
            end_ = int(round(time() * 1000)) - start
            print(f"Process Time: {end_ if end_ > 0 else 0} ms")

    return _time_it



from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends import default_backend
from cryptography.fernet import Fernet, InvalidToken
from cryptography.hazmat.primitives import hashes
import base64
import os


def getSalt():
    file_path = os.path.join(os.path.dirname(__file__), "salt.key")
    try:
        with open(file_path, 'r') as f:
            data = f.read()
            return str.encode(data)
    except FileNotFoundError:
        pass


def authenticate_key(key_password, salt=None):
    """
    This Gives You The Masterkey/Token And Does Process With key_password.And Give You The Token.
    :param key_password:
    :param salt:
    :return: Key Depending On The Key_password
    """

    if not salt:
        salt = getSalt()

    password = key_password.encode()

    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=default_backend()
    )
    key = base64.urlsafe_b64encode(kdf.derive(password))
    return key


def encrypt_content(content, key, salt=None):
    """
    Encrypts the content with the specified Alogrithm.
    :param content:
    :param key:
    :param salt:
    :return: Encrypted Content
    """

    if not salt:
        salt = getSalt()

    if content:
        masterkey = authenticate_key(key_password=key, salt=salt)
        encoded = content.encode()
        token = Fernet(masterkey)
        encrypted_content = token.encrypt(encoded)
        return encrypted_content.decode('utf-8')
    else:
        return ''


def decrypt_content(content, key, salt=None):
    """
    Decrypts The Encrypted Content.
    :param content:
    :param key:
    :param salt:
    :return: Decrypted Content.
    """

    if not salt:
        salt = getSalt()

    if content:
        masterkey = authenticate_key(key, salt=salt)
        token = Fernet(masterkey)
        decrypted_content = token.decrypt(content.encode('utf-8')).decode('UTF-8')
        return decrypted_content
    else:
        return ''


def is_valid_key(key, content, salt=None):
    """
    This Checks If The Given Key IS Valid or Not. :param key: :param salt: :param content: (sample encrypted content)
    A piece of encrypted content to validate the key.like instead of a password we use a seperate data content (in
    bytes). :return: Is Valid
    """

    if not salt:
        salt = getSalt()

    try:
        content = bytes(content.encode('UTF-8'))
        masterkey = authenticate_key(key_password=key, salt=salt)
        token = Fernet(masterkey)
        valid = token.decrypt(content)
        if valid:
            return True
    except InvalidToken:
        return False



import os
import CryptoTokenAuth
from setuptools import setup, find_packages


BASE_DIR = os.path.dirname(__file__)
version = CryptoTokenAuth.__version__


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as file:
        return file.read()


setup(
    name="CryptoTokenAuth",
    version=version,
    author="Aiden (Dev)",
    author_email="",
    long_description_content_type="text/markdown",
    long_description=read_file('README.md'),
    description='A token authentication method.',
    packages=find_packages(),
    url='https://github.com/QuackCoding/CryptoTokenAuth',
    package_data={'CryptoTokenAuth': [version]},
    install_requires=['certifi==2021.5.30', 'cffi', 'cryptography', 'pycparser'],
    keywords=['TokenAuthentication', 'CryptoTokenAuth', 'Authentication', 'Token'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
